@@ -24,11 +24,13 @@
 # modify this code, you may extend this exception to your version of the
 # code, but you are not obligated to do so.  If you do not wish to do so,
 # delete this exception statement from your version.
-
-from gi.repository import Grl, GLib, GObject
+from gi.repository import GLib, GObject
 from gnomemusic.query import Query
-from gnomemusic import log
+from gnomemusic import log, TrackerWrapper
 import logging
+import os
+os.environ['GRL_PLUGIN_RANKS'] = 'local-metadata:3,filesystem:2,tracker:1,lastfm-albumart:0'
+from gi.repository import Grl
 logger = logging.getLogger(__name__)
 
 
@@ -46,6 +48,7 @@ class Grilo(GObject.GObject):
         Grl.METADATA_KEY_DURATION,
         Grl.METADATA_KEY_CREATION_DATE,
         Grl.METADATA_KEY_URL,
+        Grl.METADATA_KEY_LYRICS,
         Grl.METADATA_KEY_THUMBNAIL]
 
     METADATA_THUMBNAIL_KEYS = [
@@ -77,8 +80,11 @@ class Grilo(GObject.GObject):
         self.changed_media_ids = []
         self.pending_event_id = 0
         self.changes_pending = {'Albums': False, 'Artists': False, 'Songs': False}
+
         self.registry = Grl.Registry.get_default()
 
+        self.sparqltracker = TrackerWrapper().tracker
+
     @log
     def _find_sources(self):
         self.registry.connect('source_added', self._on_source_added)
@@ -212,6 +218,17 @@ class Grilo(GObject.GObject):
             callback(source, param, item, remaining, data)
         self.tracker.query(query, self.METADATA_KEYS, options, _callback, data)
 
+    @log
+    def toggle_favorite(self, song_item):
+        # TODO: change "bool(song_item.get_lyrics())" --> song_item.get_favourite() once query works properly
+        # TODO: when .set/get_favourite work, set_favourite outside loop: item.set_favourite(!item.get_favourite())
+        if bool(song_item.get_lyrics()): # is favorite
+            self.sparqltracker.update(Query.remove_favorite(song_item.get_url()), GLib.PRIORITY_DEFAULT, None)
+            song_item.set_lyrics("")
+        else: # not favorite
+            self.sparqltracker.update(Query.add_favorite(song_item.get_url()), GLib.PRIORITY_DEFAULT, None)
+            song_item.set_lyrics("i'm truthy")
+
     @log
     def search(self, q, callback, data=None):
         options = self.options.copy()
