@@ -425,8 +425,13 @@ class Query():
         nmm:artistName(nmm:performer(?song)) AS artist
         nie:title(nmm:musicAlbum(?song)) AS album
         nfo:duration(?song) AS duration
+        IF(bound(?tag), 'truth!', '') AS lyrics
         {
             %(where_clause)s
+            OPTIONAL {
+                ?song nao:hasTag ?tag .
+                FILTER( ?tag = nao:predefined-tag-favorite )
+            }
             FILTER (
                 tracker:uri-is-descendant(
                     '%(music_dir)s', nie:url(?song)
@@ -500,10 +505,15 @@ class Query():
         nmm:artistName(nmm:performer(?song)) AS artist
         nie:title(nmm:musicAlbum(?song)) AS album
         nfo:duration(?song) AS duration
+        IF(bound(?tag), 'truth!', '') AS lyrics
     WHERE {
         ?song a nmm:MusicPiece ;
               a nfo:FileDataObject ;
               nmm:musicAlbum ?album .
+        OPTIONAL {
+            ?song nao:hasTag ?tag .
+            FILTER( ?tag = nao:predefined-tag-favorite )
+        }
         FILTER (
             tracker:id(?album) = %(album_id)s
         )
@@ -547,6 +557,7 @@ class Query():
         nmm:artistName(nmm:performer(?song)) AS artist
         nie:title(nmm:musicAlbum(?song)) AS album
         nfo:duration(?song) AS duration
+        IF(bound(?tag), 'truth!', '') AS lyrics
     WHERE {
         ?playlist a nmm:Playlist ;
             a nfo:MediaList ;
@@ -556,6 +567,10 @@ class Query():
         ?song a nmm:MusicPiece ;
              a nfo:FileDataObject ;
              nie:url ?url .
+        OPTIONAL {
+            ?song nao:hasTag ?tag .
+            FILTER( ?tag = nao:predefined-tag-favorite )
+        }
         FILTER (
             %(filter_clause)s
         )
@@ -958,7 +973,7 @@ class Query():
 
     @staticmethod
     def get_most_played_songs():
-        # TODO: set playlist size somewhere? Currently default is 5, this is probably too low...
+        # TODO: set playlist size somewhere? Currently default is 50.
         query = """
         SELECT ?url
         WHERE {
@@ -967,7 +982,7 @@ class Query():
                 nie:isStoredAs ?as .
           ?as nie:url ?url .
         } ORDER BY DESC(?count) LIMIT 50
-        """
+        """.replace('\n', ' ').strip()
 
         return query
 
@@ -980,33 +995,68 @@ class Query():
                 nie:isStoredAs ?as .
             ?as nie:url ?url .
             FILTER ( NOT EXISTS { ?song nie:usageCounter ?count .} )
-        } ORDER BY nfo:fileLastAccessed(?song)
+        } ORDER BY nfo:fileLastAccessed(?song) LIMIT 50
         """.replace('\n', ' ').strip()
 
         return query
 
     def get_recently_played_songs():
+            #TODO: or this could take comparison date as an argument so we don't need to make a date string in query.py...
+            #TODO: set time interval somewhere? A settings file? (Default is maybe 2 weeks...?)
+
+            days_difference = 7  # currently hardcoding time interval of 7 days
+            seconds_difference = days_difference * SECONDS_PER_DAY
+            compare_date = time.strftime(
+                sparql_midnight_dateTime_format, time.gmtime(time.time() - seconds_difference))
+
+            query = """
+            SELECT ?url
+            WHERE {
+                ?song a nmm:MusicPiece ;
+                    nie:isStoredAs ?as ;
+                    nfo:fileLastAccessed ?last_played .
+                ?as nie:url ?url .
+                FILTER ( ?last_played > '%(compare_date)s'^^xsd:dateTime )
+                FILTER ( EXISTS { ?song nie:usageCounter ?count .} )
+            } ORDER BY DESC(?last_played) LIMIT 50
+            """.replace('\n', ' ').strip() % {'compare_date': compare_date}
+
+            return query
+
+    def get_recently_added_songs():
         #TODO: or this could take comparison date as an argument so we don't need to make a date string in query.py...
         #TODO: set time interval somewhere? A settings file? (Default is maybe 2 weeks...?)
 
-        days_difference = 7  # currently hardcoding time interval of 3 days
+        days_difference = 7 # currently hardcoding time interval of 7 days
         seconds_difference = days_difference * SECONDS_PER_DAY
-        compare_date = time.strftime(
-            sparql_midnight_dateTime_format, time.gmtime(time.time() - seconds_difference))
+        compare_date = time.strftime(sparql_midnight_dateTime_format, time.gmtime(time.time()-seconds_difference))
 
         query = """
         SELECT ?url
         WHERE {
             ?song a nmm:MusicPiece ;
                 nie:isStoredAs ?as ;
-                nfo:fileLastAccessed ?last_played .
+                tracker:added ?added .
             ?as nie:url ?url .
-            FILTER ( ?last_played > '%(compare_date)s'^^xsd:dateTime )
-        } ORDER BY DESC(?last_played)
+            FILTER ( ?added > '%(compare_date)s'^^xsd:dateTime )
+        } ORDER BY DESC(?added) LIMIT 50
         """.replace('\n', ' ').strip() % {'compare_date': compare_date}
 
         return query
 
+    def get_favorite_songs():
+        query = """
+    SELECT ?url
+    WHERE {
+        ?song a nmm:MusicPiece ;
+            nie:isStoredAs ?as ;
+            nao:hasTag nao:predefined-tag-favorite .
+        ?as nie:url ?url .
+    } ORDER BY DESC(tracker:added(?song))
+    """.replace('\n', ' ').strip()
+
+        return query
+
     # Functions for search
     # TODO: make those queries actually return something
     @staticmethod
@@ -1269,3 +1319,34 @@ class Query():
         }
 
         return query
+
+    def add_favorite(song_url):
+        query = """
+            INSERT {
+                ?song nao:hasTag nao:predefined-tag-favorite
+            }
+            WHERE {
+                ?song a nmm:MusicPiece .
+                FILTER ( nie:url(?song) = "%(song_url)s" )
+            }
+        """.replace("\n", " ").strip() % {
+            'song_url': song_url
+
+        }
+
+        return query
+
+    def remove_favorite(song_url):
+        query = """
+            DELETE {
+                ?song nao:hasTag nao:predefined-tag-favorite
+            }
+            WHERE {
+                ?song a nmm:MusicPiece .
+                FILTER ( nie:url(?song) = "%(song_url)s" )
+            }
+        """.replace("\n", " ").strip() % {
+            'song_url': song_url
+        }
+
+        return query
\ No newline at end of file
