@@ -35,6 +35,7 @@ from gi.repository import Gtk
 from gi.repository import GObject
 from gi.repository import Gd
 from gi.repository import Grl
+from gi.repository import Gio
 from gi.repository import Pango
 from gi.repository import GLib
 from gi.repository import GdkPixbuf
@@ -44,7 +45,8 @@ from gnomemusic.grilo import grilo
 from gnomemusic.query import Query
 from gnomemusic.toolbar import ToolbarState
 import gnomemusic.widgets as Widgets
-from gnomemusic.playlists import Playlists
+from gnomemusic.player import DiscoveryStatus
+from gnomemusic.playlists import Playlists, StaticPlaylists
 from gnomemusic.albumArtCache import AlbumArtCache as albumArtCache
 from gnomemusic import log
 import logging
@@ -56,7 +58,6 @@ playlists = Playlists.get_default()
 class ViewContainer(Gtk.Stack):
     nowPlayingIconName = 'media-playback-start-symbolic'
     errorIconName = 'dialog-error-symbolic'
-    starIconName = 'starred-symbolic'
 
     @log
     def __init__(self, name, title, window, view_type, use_sidebar=False, sidebar=None):
@@ -81,7 +82,8 @@ class ViewContainer(Gtk.Stack):
             GObject.TYPE_INT,
             GObject.TYPE_STRING,
             GObject.TYPE_BOOLEAN,
-            GObject.TYPE_BOOLEAN
+            GObject.TYPE_BOOLEAN,
+            GObject.TYPE_INT
         )
         self.view = Gd.MainView(
             shadow_type=Gtk.ShadowType.NONE
@@ -105,7 +107,8 @@ class ViewContainer(Gtk.Stack):
         if not use_sidebar or sidebar:
             self._grid.add(box)
 
-        self.view.connect('item-activated', self._on_item_activated)
+        self.view.click_handler = self.view.connect('item-activated', self._on_item_activated)
+        self.star_renderer_click = False
         self.view.connect('selection-mode-request', self._on_selection_mode_request)
         self._cursor = None
         self.window = window
@@ -205,19 +208,11 @@ class ViewContainer(Gtk.Stack):
     def populate(self):
         print('populate')
 
-    @log
-    def _on_discovered(self, info, error, _iter):
-        if error:
-            try:
-                logger.warn("File will be skipped: %s\n%s" % (error.message, info.get_uri()))
-                self._model.set(_iter, [8, 10], [self.errorIconName, True])
-            except Exception:
-                pass
-
     @log
     def _add_item(self, source, param, item, remaining=0, data=None):
         if not item:
             if remaining == 0:
+                self.window.notification.dismiss()
                 self.view.show()
             return
         self._offset += 1
@@ -229,18 +224,11 @@ class ViewContainer(Gtk.Stack):
 
         def add_new_item():
             _iter = self._model.append(None)
-            icon_name = self.nowPlayingIconName
-            if item.get_url():
-                try:
-                    self.player.discoverer.discover_uri(item.get_url())
-                except:
-                    logger.warn('failed to discover url ' + item.get_url())
-                    icon_name = self.errorIconName
             self._model.set(_iter,
-                            [0, 1, 2, 3, 4, 5, 7, 8, 9, 10],
+                            [0, 1, 2, 3, 4, 5, 7, 9],
                             [str(item.get_id()), '', title,
                              artist, self._loadingIcon, item,
-                             0, icon_name, False, icon_name == self.errorIconName])
+                             0, False])
             self.cache.lookup(item, self._iconWidth, self._iconHeight, self._on_lookup_ready,
                               _iter, artist, title)
         GLib.idle_add(add_new_item)
@@ -267,6 +255,25 @@ class ViewContainer(Gtk.Stack):
     def get_selected_tracks(self, callback):
         callback([])
 
+    def _on_list_widget_star_render(self, col, cell, model, _iter, data):
+        pass
+
+    @log
+    def _on_star_toggled(self, widget, path):
+        try:
+            _iter = self._model.get_iter(path)
+        except TypeError:
+            return
+
+        new_value = not self._model.get_value(_iter, 9)
+        self._model.set_value(_iter, 9, new_value)
+        song_item = self._model.get_value(_iter, 5) # er, will this definitely return MediaAudio obj.?
+        grilo.toggle_favorite(song_item) # toggle favorite status in database
+        playlists.update_static_playlist(StaticPlaylists.Favorites)
+
+        # Use this flag to ignore the upcoming _on_item_activated call
+        self.star_renderer_click = True
+
 
 # Class for the Empty View
 class Empty(Gtk.Stack):
@@ -291,6 +298,7 @@ class Albums(ViewContainer):
         self.albums_selected = []
         self.items_selected = []
         self.items_selected_callback = None
+        self._add_list_renderers()
 
     @log
     def _on_changes_pending(self, data=None):
@@ -311,6 +319,10 @@ class Albums(ViewContainer):
 
     @log
     def _on_item_activated(self, widget, id, path):
+        if self.star_renderer_click:
+            self.star_renderer_click = False
+            return
+
         try:
             _iter = self._model.get_iter(path)
         except TypeError:
@@ -329,6 +341,7 @@ class Albums(ViewContainer):
     @log
     def populate(self):
         if grilo.tracker:
+            self.window._init_loading_notification()
             GLib.idle_add(grilo.populate_albums, self._offset, self._add_item)
 
     @log
@@ -396,12 +409,16 @@ class Songs(ViewContainer):
 
     @log
     def _on_item_activated(self, widget, id, path):
+        if self.star_renderer_click:
+            self.star_renderer_click = False
+            return
+
         try:
             _iter = self._model.get_iter(path)
         except TypeError:
             return
         if self._model.get_value(_iter, 8) != self.errorIconName:
-            self.player.set_playlist('Songs', None, self._model, _iter, 5)
+            self.player.set_playlist('Songs', None, self._model, _iter, 5, 11)
             self.player.set_playing(True)
 
     @log
@@ -421,6 +438,7 @@ class Songs(ViewContainer):
     def _add_item(self, source, param, item, remaining=0, data=None):
         if not item:
             if remaining == 0:
+                self.window.notification.dismiss()
                 self.view.show()
             return
         self._offset += 1
@@ -432,10 +450,10 @@ class Songs(ViewContainer):
             return
         _iter = self._model.insert_with_valuesv(
             -1,
-            [2, 3, 5, 8, 9, 10],
+            [2, 3, 5, 9],
             [albumArtCache.get_media_title(item),
-             artist, item, self.nowPlayingIconName, False, False])
-        self.player.discover_item(item, self._on_discovered, _iter)
+             artist, item, bool(item.get_lyrics())])
+        # TODO: change "bool(item.get_lyrics())" --> item.get_favourite() once query works properly
 
     @log
     def _add_list_renderers(self):
@@ -448,10 +466,8 @@ class Songs(ViewContainer):
         column_now_playing = Gtk.TreeViewColumn()
         column_now_playing.set_property('fixed_width', 24)
         column_now_playing.pack_start(now_playing_symbol_renderer, False)
-        column_now_playing.add_attribute(now_playing_symbol_renderer,
-                                         'visible', 10)
-        column_now_playing.add_attribute(now_playing_symbol_renderer,
-                                         'icon_name', 8)
+        column_now_playing.set_cell_data_func(now_playing_symbol_renderer,
+                                              self._on_list_widget_icon_render, None)
         list_widget.insert_column(column_now_playing, 0)
 
         title_renderer = Gtk.CellRendererText(
@@ -465,13 +481,12 @@ class Songs(ViewContainer):
                                  self._on_list_widget_title_render, None)
         cols[0].add_attribute(title_renderer, 'text', 2)
 
-        star_renderer = Gtk.CellRendererPixbuf(
-            xpad=32,
-            icon_name=self.starIconName
-        )
+        # ADD STAR RENDERERS
+        star_renderer = Widgets.CellRendererClickablePixbuf(self.view)
+        star_renderer.connect("clicked", self._on_star_toggled)
         list_widget.add_renderer(star_renderer,
-                                 self._on_list_widget_star_render, None)
-        cols[0].add_attribute(star_renderer, 'visible', 9)
+                                self._on_list_widget_star_render, None)
+        cols[0].add_attribute(star_renderer, 'show_star', 9)
 
         duration_renderer = Gd.StyledTextRenderer(
             xpad=32,
@@ -501,9 +516,6 @@ class Songs(ViewContainer):
     def _on_list_widget_title_render(self, col, cell, model, _iter, data):
         pass
 
-    def _on_list_widget_star_render(self, col, cell, model, _iter, data):
-        pass
-
     def _on_list_widget_duration_render(self, col, cell, model, _iter, data):
         item = model.get_value(_iter, 5)
         if item:
@@ -520,10 +532,25 @@ class Songs(ViewContainer):
         if item:
             cell.set_property('text', item.get_string(Grl.METADATA_KEY_ALBUM) or _("Unknown Album"))
 
+    def _on_list_widget_icon_render(self, col, cell, model, _iter, data):
+        if model != self.player.playlist:
+            cell.set_visible(False)
+            return
+
+        if model.get_value(_iter, 11) == DiscoveryStatus.FAILED:
+            cell.set_property('icon-name', self.errorIconName)
+            cell.set_visible(True)
+        elif model.get_path(_iter) == self.player.currentTrack.get_path():
+            cell.set_property('icon-name', self.nowPlayingIconName)
+            cell.set_visible(True)
+        else:
+            cell.set_visible(False)
+
     @log
     def populate(self):
         self._init = True
         if grilo.tracker:
+            self.window._init_loading_notification()
             GLib.idle_add(grilo.populate_songs, self._offset, self._add_item)
 
     @log
@@ -553,11 +580,11 @@ class Artists (ViewContainer):
         self.artistAlbumsStack.add_named(self._artistAlbumsWidget, "sidebar")
         self.artistAlbumsStack.set_visible_child_name("sidebar")
         self.view.set_hexpand(False)
-        self.view.get_style_context().add_class('artist-panel')
         self.view.get_generic_view().get_selection().set_mode(
             Gtk.SelectionMode.SINGLE)
         self._grid.attach(self.artistAlbumsStack, 2, 0, 2, 2)
         self._add_list_renderers()
+        self.view.get_generic_view().get_style_context().remove_class('content-view')
         if (Gtk.Settings.get_default().get_property(
                 'gtk_application_prefer_dark_theme')):
             self.view.get_generic_view().get_style_context().\
@@ -585,7 +612,7 @@ class Artists (ViewContainer):
             self._last_selection = self._allIter
             self._artists[_("All Artists").casefold()] =\
                 {'iter': self._allIter, 'albums': [], 'widget': None}
-            selection.select_path(self._model.get_path(self._allIter))
+            #selection.select_path(self._model.get_path(self._allIter))
         self._init = True
         self.populate()
 
@@ -615,6 +642,10 @@ class Artists (ViewContainer):
 
     @log
     def _on_item_activated(self, widget, item_id, path):
+        if self.star_renderer_click:
+            self.star_renderer_click = False
+            return
+
         try:
             _iter = self._model.get_iter(path)
         except TypeError:
@@ -646,12 +677,12 @@ class Artists (ViewContainer):
         if (self._model.get_string_from_iter(_iter) ==
                 self._model.get_string_from_iter(self._allIter)):
             artistAlbums = Widgets.AllArtistsAlbums(
-                self.player, self.header_bar, self.selection_toolbar
+                self.player, self.header_bar, self.selection_toolbar, self.window
             )
         else:
             artistAlbums = Widgets.ArtistAlbums(
                 artist, albums, self.player,
-                self.header_bar, self.selection_toolbar
+                self.header_bar, self.selection_toolbar, self.window
             )
         self._artists[artist.casefold()]['widget'] = artistAlbums
         new_artistAlbumsWidget.add(artistAlbums)
@@ -665,7 +696,7 @@ class Artists (ViewContainer):
     def _add_item(self, source, param, item, remaining=0, data=None):
         if item is None:
             if remaining == 0:
-                self.add_all_artists_entry()
+                self.window.notification.dismiss()
                 self.view.show()
             return
         self._offset += 1
@@ -681,6 +712,7 @@ class Artists (ViewContainer):
     @log
     def populate(self):
         if grilo.tracker:
+            self.window._init_loading_notification()
             GLib.idle_add(grilo.populate_artists, self._offset, self._add_item)
 
     @log
@@ -768,10 +800,12 @@ class Playlist(ViewContainer):
         self.name_label = builder.get_object('playlist_name')
         self.songs_count_label = builder.get_object('songs_count')
         self.menubutton = builder.get_object('playlist_menubutton')
-        self.play_menuitem = builder.get_object('menuitem_play')
-        self.play_menuitem.connect('activate', self._on_play_activate)
-        self.delete_menuitem = builder.get_object('menuitem_delete')
-        self.delete_menuitem.connect('activate', self._on_delete_activate)
+        playlistPlayAction = Gio.SimpleAction.new('playlist_play', None)
+        playlistPlayAction.connect('activate', self._on_play_activate)
+        window.add_action(playlistPlayAction)
+        playlistDeleteAction = Gio.SimpleAction.new('playlist_delete', None)
+        playlistDeleteAction.connect('activate', self._on_delete_activate)
+        window.add_action(playlistDeleteAction)
         self._grid.insert_row(0)
         self._grid.attach(self.headerbar, 1, 0, 1, 1)
 
@@ -786,13 +820,14 @@ class Playlist(ViewContainer):
             GObject.TYPE_INT,
             GObject.TYPE_STRING,
             GObject.TYPE_BOOLEAN,
-            GObject.TYPE_BOOLEAN
+            GObject.TYPE_BOOLEAN,
+            GObject.TYPE_INT
         )
 
         self.playlists_sidebar.set_view_type(Gd.MainViewType.LIST)
         self.playlists_sidebar.set_model(self.playlists_model)
         self.playlists_sidebar.set_hexpand(False)
-        self.playlists_sidebar.get_style_context().add_class('artist-panel')
+        #self.playlists_sidebar.get_style_context().add_class('artist-panel')
         self.playlists_sidebar.get_generic_view().get_selection().set_mode(
             Gtk.SelectionMode.SINGLE)
         self.playlists_sidebar.connect('item-activated', self._on_playlist_activated)
@@ -800,6 +835,7 @@ class Playlist(ViewContainer):
         self._grid.child_set_property(self.stack, 'top-attach', 0)
         self._grid.child_set_property(self.stack, 'height', 2)
         self._add_sidebar_renderers()
+        self.playlists_sidebar.get_generic_view().get_style_context().remove_class('content-view')
         if (Gtk.Settings.get_default().get_property(
                 'gtk_application_prefer_dark_theme')):
             self.playlists_sidebar.get_generic_view().get_style_context().\
@@ -824,6 +860,10 @@ class Playlist(ViewContainer):
         playlists.connect('song-removed-from-playlist', self._on_song_removed_from_playlist)
         self.show_all()
 
+    @log
+    def _on_changes_pending(self, data=None):
+        playlists.update_all_static_playlists()
+
     @log
     def _add_list_renderers(self):
         list_widget = self.view.get_generic_view()
@@ -835,10 +875,8 @@ class Playlist(ViewContainer):
         column_now_playing = Gtk.TreeViewColumn()
         column_now_playing.set_property('fixed_width', 24)
         column_now_playing.pack_start(now_playing_symbol_renderer, False)
-        column_now_playing.add_attribute(now_playing_symbol_renderer,
-                                         'visible', 10)
-        column_now_playing.add_attribute(now_playing_symbol_renderer,
-                                         'icon_name', 8)
+        column_now_playing.set_cell_data_func(now_playing_symbol_renderer,
+                                              self._on_list_widget_icon_render, None)
         list_widget.insert_column(column_now_playing, 0)
 
         title_renderer = Gtk.CellRendererText(
@@ -852,13 +890,12 @@ class Playlist(ViewContainer):
                                  self._on_list_widget_title_render, None)
         cols[0].add_attribute(title_renderer, 'text', 2)
 
-        star_renderer = Gtk.CellRendererPixbuf(
-            xpad=32,
-            icon_name=self.starIconName
-        )
+        # ADD STAR RENDERERS
+        star_renderer = Widgets.CellRendererClickablePixbuf(self.view)
+        star_renderer.connect("clicked", self._on_star_toggled)
         list_widget.add_renderer(star_renderer,
-                                 self._on_list_widget_star_render, None)
-        cols[0].add_attribute(star_renderer, 'visible', 9)
+                                self._on_list_widget_star_render, None)
+        cols[0].add_attribute(star_renderer, 'show_star', 9)
 
         duration_renderer = Gd.StyledTextRenderer(
             xpad=32,
@@ -926,9 +963,24 @@ class Playlist(ViewContainer):
         if item:
             cell.set_property('text', item.get_string(Grl.METADATA_KEY_ALBUM) or _("Unknown Album"))
 
+    def _on_list_widget_icon_render(self, col, cell, model, _iter, data):
+        if model != self.player.playlist:
+            cell.set_visible(False)
+            return
+
+        if model.get_value(_iter, 11) == DiscoveryStatus.FAILED:
+            cell.set_property('icon-name', self.errorIconName)
+            cell.set_visible(True)
+        elif model.get_path(_iter) == self.player.currentTrack.get_path():
+            cell.set_property('icon-name', self.nowPlayingIconName)
+            cell.set_visible(True)
+        else:
+            cell.set_visible(False)
+
     @log
     def _populate(self):
         self._init = True
+        self.window._init_loading_notification()
         self.populate()
 
     @log
@@ -951,6 +1003,7 @@ class Playlist(ViewContainer):
     @log
     def _add_playlist_item_to_model(self, item):
         if not item:
+            self.window.notification.dismiss()
             self.emit('playlists-loaded')
             return
         _iter = self.playlists_model.insert_with_valuesv(
@@ -966,6 +1019,10 @@ class Playlist(ViewContainer):
 
     @log
     def _on_item_activated(self, widget, id, path):
+        if self.star_renderer_click:
+            self.star_renderer_click = False
+            return
+
         try:
             _iter = self._model.get_iter(path)
         except TypeError:
@@ -973,7 +1030,7 @@ class Playlist(ViewContainer):
         if self._model.get_value(_iter, 8) != self.errorIconName:
             self.player.set_playlist(
                 'Playlist', self.current_playlist.get_id(),
-                self._model, _iter, 5
+                self._model, _iter, 5, 11
             )
             self.player.set_playing(True)
 
@@ -1068,7 +1125,8 @@ class Playlist(ViewContainer):
                 GObject.TYPE_INT,
                 GObject.TYPE_STRING,
                 GObject.TYPE_BOOLEAN,
-                GObject.TYPE_BOOLEAN
+                GObject.TYPE_BOOLEAN,
+                GObject.TYPE_INT
             )
             self.view.set_model(self._model)
             GLib.idle_add(grilo.populate_playlist_songs, playlist, self._add_item)
@@ -1092,9 +1150,8 @@ class Playlist(ViewContainer):
             or _("Unknown Artist")
         _iter = model.insert_with_valuesv(
             -1,
-            [2, 3, 5, 8, 9, 10],
-            [title, artist, item, self.nowPlayingIconName, False, False])
-        self.player.discover_item(item, self._on_discovered, _iter)
+            [2, 3, 5, 9],
+            [title, artist, item, bool(item.get_lyrics())])
         self.songs_count += 1
         self._update_songs_count()
 
@@ -1145,7 +1202,7 @@ class Playlist(ViewContainer):
 
     @log
     def _on_delete_activate(self, menuitem, data=None):
-        self.window._init_notification()
+        self.window._init_playlist_removal_notification()
         self.delete_selected_playlist()
 
     @log
@@ -1209,7 +1266,7 @@ class Playlist(ViewContainer):
 
                     self.iter_to_clean = None
                     self.update_model(self.player, model, nextIter)
-                    self.player.set_playlist('Playlist', playlist.get_id(), model, nextIter, 5)
+                    self.player.set_playlist('Playlist', playlist.get_id(), model, nextIter, 5, 11)
                     self.player.set_playing(True)
 
                 # Update songs count
@@ -1270,6 +1327,10 @@ class Search(ViewContainer):
 
     @log
     def _on_item_activated(self, widget, id, path):
+        if self.star_renderer_click:
+            self.star_renderer_click = False
+            return
+
         try:
             child_path = self.filter_model.convert_path_to_child_path(path)
         except TypeError:
@@ -1293,7 +1354,7 @@ class Search(ViewContainer):
 
             self._artistAlbumsWidget = Widgets.ArtistAlbums(
                 artist, albums, self.player,
-                self.header_bar, self.selection_toolbar, True
+                self.header_bar, self.selection_toolbar, self.window, True
             )
             self.add(self._artistAlbumsWidget)
 
@@ -1302,9 +1363,9 @@ class Search(ViewContainer):
             self.set_visible_child(self._artistAlbumsWidget)
             self.header_bar.searchbar.show_bar(False)
         elif self._model[_iter][11] == 'song':
-            if self._model.get_value(_iter, 8) != self.errorIconName:
+            if self._model.get_value(_iter, 12) != DiscoveryStatus.FAILED:
                 child_iter = self.songs_model.convert_child_iter_to_iter(_iter)[1]
-                self.player.set_playlist('Search Results', None, self.songs_model, child_iter, 5)
+                self.player.set_playlist('Search Results', None, self.songs_model, child_iter, 5, 12)
                 self.player.set_playing(True)
         else:  # Headers
             if self.view.get_generic_view().row_expanded(path):
@@ -1347,6 +1408,7 @@ class Search(ViewContainer):
             return
 
         if remaining == 0:
+            self.window.notification.dismiss()
             self.view.show()
 
         model, category = data
@@ -1370,36 +1432,30 @@ class Search(ViewContainer):
         if category == 'album':
             _iter = self._model.insert_with_values(
                 self.head_iters[group], -1,
-                [0, 2, 3, 4, 5, 8, 9, 10, 11],
+                [0, 2, 3, 4, 5, 9, 11],
                 [str(item.get_id()), title, artist,
-                 self._loadingIcon, item, self.nowPlayingIconName,
-                 False, False, category])
+                 self._loadingIcon, item, False, category])
             self.cache.lookup(item, self._iconWidth, self._iconHeight, self._on_lookup_ready,
                               _iter, artist, title)
         elif category == 'song':
             _iter = self._model.insert_with_values(
                 self.head_iters[group], -1,
-                [0, 2, 3, 4, 5, 8, 9, 10, 11],
+                [0, 2, 3, 4, 5, 9, 11],
                 [str(item.get_id()), title, artist,
-                 self._noAlbumArtIcon, item, self.nowPlayingIconName,
-                 False, False, category])
+                 self._noAlbumArtIcon, item, bool(item.get_lyrics()), category])
         else:
             if not artist.casefold() in self._artists:
                 _iter = self._model.insert_with_values(
                     self.head_iters[group], -1,
-                    [0, 2, 4, 5, 8, 9, 10, 11],
+                    [0, 2, 4, 5, 9, 11],
                     [str(item.get_id()), artist,
-                     self._loadingIcon, item, self.nowPlayingIconName,
-                     False, False, category])
+                     self._loadingIcon, item, False, category])
                 self.cache.lookup(item, self._iconWidth, self._iconHeight, self._on_lookup_ready,
                                   _iter, artist, title)
                 self._artists[artist.casefold()] = {'iter': _iter, 'albums': []}
 
             self._artists[artist.casefold()]['albums'].append(item)
 
-        if category == 'song':
-            self.player.discover_item(item, self._on_discovered, _iter)
-
         if self._model.iter_n_children(self.head_iters[group]) == 1:
             path = self._model.get_path(self.head_iters[group])
             self.view.get_generic_view().expand_row(path, False)
@@ -1421,6 +1477,12 @@ class Search(ViewContainer):
                                  self._on_list_widget_title_render, None)
         cols[0].add_attribute(title_renderer, 'text', 2)
 
+        star_renderer = Widgets.CellRendererClickablePixbuf(self.view, hidden=True)
+        star_renderer.connect("clicked", self._on_star_toggled)
+        list_widget.add_renderer(star_renderer,
+                                 self._on_list_widget_star_render, None)
+        cols[0].add_attribute(star_renderer, 'show_star', 9)
+
         cells = cols[0].get_cells()
         cols[0].reorder(cells[0], -1)
         cols[0].set_cell_data_func(cells[0], self._on_list_widget_selection_render, None)
@@ -1436,6 +1498,7 @@ class Search(ViewContainer):
 
     @log
     def populate(self):
+        self.window._init_loading_notification()
         self.header_bar.set_state(ToolbarState.MAIN)
 
     @log
@@ -1571,7 +1634,8 @@ class Search(ViewContainer):
             GObject.TYPE_STRING,
             GObject.TYPE_BOOLEAN,
             GObject.TYPE_BOOLEAN,
-            GObject.TYPE_STRING     # type
+            GObject.TYPE_STRING,    # type
+            GObject.TYPE_INT
         )
         self.filter_model = self._model.filter_new(None)
         self.filter_model.set_visible_func(self._filter_visible_func)
