@@ -36,7 +36,8 @@ from gi.repository import GdkPixbuf, Grl
 from gettext import gettext as _, ngettext
 from gnomemusic.grilo import grilo
 from gnomemusic.albumArtCache import AlbumArtCache
-from gnomemusic.playlists import Playlists
+from gnomemusic.player import DiscoveryStatus
+from gnomemusic.playlists import Playlists, StaticPlaylists
 from gnomemusic import log
 import logging
 logger = logging.getLogger(__name__)
@@ -52,6 +53,8 @@ except Exception as e:
     MAX_TITLE_WIDTH = 20
     logger.error("Error on setting widget max-width-chars: %s" % str(e))
 
+playlists = Playlists.get_default()
+
 
 class AlbumWidget(Gtk.EventBox):
 
@@ -88,7 +91,7 @@ class AlbumWidget(Gtk.EventBox):
         view_box.add(child_view)
         self.add(self.ui.get_object('AlbumWidget'))
         self._add_list_renderers()
-        # TODO: make this work
+        self.star_renderer_click = False
         self.get_style_context().add_class('view')
         self.get_style_context().add_class('content-view')
         self.show_all()
@@ -99,16 +102,20 @@ class AlbumWidget(Gtk.EventBox):
 
     @log
     def _on_item_activated(self, widget, id, path):
+        if self.star_renderer_click:
+            self.star_renderer_click = False
+            return
+
         _iter = self.model.get_iter(path)
 
-        if(self.model.get_value(_iter, 7) != ERROR_ICON_NAME):
+        if self.model.get_value(_iter, 10) != DiscoveryStatus.FAILED:
             if (self.iterToClean and self.player.playlistId == self.album):
                 item = self.model.get_value(self.iterToClean, 5)
                 title = AlbumArtCache.get_media_title(item)
                 self.model.set_value(self.iterToClean, 0, title)
                 # Hide now playing icon
                 self.model.set_value(self.iterToClean, 6, False)
-            self.player.set_playlist('Album', self.album, self.model, _iter, 5)
+            self.player.set_playlist('Album', self.album, self.model, _iter, 5, 11)
             self.player.set_playing(True)
 
     @log
@@ -129,10 +136,8 @@ class AlbumWidget(Gtk.EventBox):
         column_now_playing = Gtk.TreeViewColumn()
         column_now_playing.set_fixed_width(24)
         column_now_playing.pack_start(now_playing_symbol_renderer, False)
-        column_now_playing.add_attribute(now_playing_symbol_renderer,
-                                         'visible', 9)
-        column_now_playing.add_attribute(now_playing_symbol_renderer,
-                                         'icon_name', 7)
+        column_now_playing.set_cell_data_func(now_playing_symbol_renderer,
+                                              self._on_list_widget_icon_render, None)
         list_widget.insert_column(column_now_playing, 0)
 
         type_renderer = Gd.StyledTextRenderer(
@@ -154,6 +159,26 @@ class AlbumWidget(Gtk.EventBox):
         cols[0].clear_attributes(durationRenderer)
         cols[0].add_attribute(durationRenderer, 'markup', 1)
 
+        star_renderer = CellRendererClickablePixbuf(self.view)
+        star_renderer.connect("clicked", self._on_star_toggled)
+        list_widget.add_renderer(star_renderer, lambda *args: None, None)
+        cols[0].clear_attributes(star_renderer)
+        cols[0].add_attribute(star_renderer, 'show_star', 10)
+
+    def _on_list_widget_icon_render(self, col, cell, model, _iter, data):
+        if model != self.player.playlist:
+            cell.set_visible(False)
+            return
+
+        if model.get_value(_iter, 10) == DiscoveryStatus.FAILED:
+            cell.set_property('icon-name', ERROR_ICON_NAME)
+            cell.set_visible(True)
+        elif model.get_path(_iter) == self.player.currentTrack.get_path():
+            cell.set_property('icon-name', NOW_PLAYING_ICON_NAME)
+            cell.set_visible(True)
+        else:
+            cell.set_visible(False)
+
     @log
     def _create_model(self):
         self.model = Gtk.ListStore(
@@ -167,6 +192,8 @@ class AlbumWidget(Gtk.EventBox):
             GObject.TYPE_STRING,
             GObject.TYPE_BOOLEAN,
             GObject.TYPE_BOOLEAN,  # icon shown
+            GObject.TYPE_BOOLEAN,
+            GObject.TYPE_INT
         )
 
     @log
@@ -244,26 +271,19 @@ class AlbumWidget(Gtk.EventBox):
             if(self.player.get_playback_status() != 2):
                 self.player.actionbar.set_visible(True)
 
-    @log
-    def _on_discovered(self, info, error, _iter):
-        if error:
-            self.model.set(_iter, [7, 9], [ERROR_ICON_NAME, True])
-
     @log
     def add_item(self, source, prefs, track, remaining, data=None):
         if track:
             self.tracks.append(track)
             self.duration = self.duration + track.get_duration()
             _iter = self.model.append()
-            self.player.discover_item(track, self._on_discovered, _iter)
             escapedTitle = AlbumArtCache.get_media_title(track, True)
             self.model.set(_iter,
-                           [0, 1, 2, 3, 4, 5, 7, 9],
+                           [0, 1, 2, 3, 4, 5, 10],
                            [escapedTitle,
                             self.player.seconds_to_string(
                                 track.get_duration()),
-                            '', '', None, track, NOW_PLAYING_ICON_NAME,
-                            False])
+                            '', '', None, track, bool(track.get_lyrics())])
             self.ui.get_object('running_length_label_info').set_text(
                 _("%d min") % (int(self.duration / 60) + 1))
 
@@ -291,32 +311,44 @@ class AlbumWidget(Gtk.EventBox):
             escapedTitle = AlbumArtCache.get_media_title(song, True)
             if (song == currentSong):
                 title = '<b>%s</b>' % escapedTitle
-                iconVisible = True
                 song_passed = True
             elif (song_passed):
                 title = '<span>%s</span>' % escapedTitle
-                iconVisible = False
             else:
                 title = '<span color=\'grey\'>%s</span>' % escapedTitle
-                iconVisible = False
             playlist.set_value(_iter, 0, title)
-            if(playlist.get_value(_iter, 7) != ERROR_ICON_NAME):
-                playlist.set_value(_iter, 9, iconVisible)
             _iter = playlist.iter_next(_iter)
             self.ui.get_object('running_length_label_info').set_text(
                 _("%d min") % (int(self.duration / 60) + 1))
         return False
 
+    @log
+    def _on_star_toggled(self, widget, path):
+        try:
+            _iter = self.model.get_iter(path)
+        except TypeError:
+            return
+
+        new_value = not self.model.get_value(_iter, 10)
+        self.model.set_value(_iter, 10, new_value)
+        song_item = self.model.get_value(_iter, 5) # er, will this definitely return MediaAudio obj.?
+        grilo.toggle_favorite(song_item) # toggle favorite status in database
+        playlists.update_static_playlist(StaticPlaylists.Favorites)
+
+        # Use this flag to ignore the upcoming _on_item_activated call
+        self.star_renderer_click = True
+
 
 class ArtistAlbums(Gtk.Box):
 
     @log
     def __init__(self, artist, albums, player,
-                 header_bar, selection_toolbar, selectionModeAllowed=False):
+                 header_bar, selection_toolbar, window, selectionModeAllowed=False):
         Gtk.Box.__init__(self, orientation=Gtk.Orientation.VERTICAL)
         self.player = player
         self.artist = artist
         self.albums = albums
+        self.window = window
         self.selectionMode = False
         self.selectionModeAllowed = selectionModeAllowed
         self.selection_toolbar = selection_toolbar
@@ -333,7 +365,8 @@ class ArtistAlbums(Gtk.Box):
                                    GObject.TYPE_BOOLEAN,  # icon shown
                                    GObject.TYPE_STRING,   # icon
                                    GObject.TYPE_OBJECT,   # song object
-                                   GObject.TYPE_BOOLEAN
+                                   GObject.TYPE_BOOLEAN,
+                                   GObject.TYPE_INT
                                    )
         self.model.connect('row-changed', self._model_row_changed)
 
@@ -353,6 +386,7 @@ class ArtistAlbums(Gtk.Box):
         self.pack_start(self._scrolledWindow, True, True, 0)
 
         self.hide()
+        self.window._init_loading_notification()
 
         for album in albums:
             is_last_album = False
@@ -363,6 +397,7 @@ class ArtistAlbums(Gtk.Box):
         self.player.connect('playlist-item-changed', self.update_model)
 
     def _on_last_album_displayed(self, data=None):
+        self.window.notification.dismiss()
         self.show_all()
 
     @log
@@ -462,20 +497,21 @@ class AllArtistsAlbums(ArtistAlbums):
                               header_bar, selection_toolbar, selectionModeAllowed)
         self._offset = 0
         self._populate()
-        self.show()
 
     @log
     def _populate(self, data=None):
         if grilo.tracker:
             GLib.idle_add(grilo.populate_albums,
-                          self._offset, self.add_item, -1)
+                          self._offset, self.add_item)
 
     @log
     def add_item(self, source, param, item, remaining=0, data=None):
+        if remaining == 0:
+            self._on_last_album_displayed()
+
         if item:
             self._offset += 1
             self.add_album(item)
-            self.show_all()
 
 
 class ArtistAlbumWidget(Gtk.Box):
@@ -517,16 +553,6 @@ class ArtistAlbumWidget(Gtk.Box):
         grilo.populate_album_songs(album, self.add_item)
         self.pack_start(self.ui.get_object('ArtistAlbumWidget'), True, True, 0)
 
-    @log
-    def _on_discovered(self, info, error, song_widget):
-        if error:
-            self.model.set(song_widget._iter, [4], [ERROR_ICON_NAME])
-            song_widget.now_playing_sign.set_from_icon_name(
-                ERROR_ICON_NAME,
-                Gtk.IconSize.SMALL_TOOLBAR)
-            song_widget.now_playing_sign.show()
-            song_widget.can_be_played = False
-
     @log
     def add_item(self, source, prefs, track, remaining, data=None):
         if remaining == 0:
@@ -564,11 +590,9 @@ class ArtistAlbumWidget(Gtk.Box):
                 song_widget.checkButton.connect(
                     'toggled', self._check_button_toggled, song_widget
                 )
-                self.player.discover_item(track, self._on_discovered, song_widget)
                 self.model.set(itr,
-                               [0, 1, 2, 3, 4, 5],
-                               [title, '', '', False,
-                                NOW_PLAYING_ICON_NAME, track])
+                               [0, 1, 2, 3, 5],
+                               [title, '', '', False, track])
                 song_widget.now_playing_sign = ui.get_object('image1')
                 song_widget.now_playing_sign.set_from_icon_name(
                     NOW_PLAYING_ICON_NAME,
@@ -613,7 +637,7 @@ class ArtistAlbumWidget(Gtk.Box):
 
         self.player.stop()
         self.player.set_playlist('Artist', self.artist,
-                                 widget.model, widget._iter, 5)
+                                 widget.model, widget._iter, 5, 6)
         self.player.set_playing(True)
 
     @log
@@ -639,6 +663,14 @@ class ArtistAlbumWidget(Gtk.Box):
             return
         songWidget = model[_iter][5].song_widget
         selected = model[_iter][6]
+
+        if model[_iter][11] == DiscoveryStatus.FAILED:
+            songWidget.now_playing_sign.set_from_icon_name(
+                ERROR_ICON_NAME,
+                Gtk.IconSize.SMALL_TOOLBAR)
+            songWidget.now_playing_sign.show()
+            songWidget.can_be_played = False
+
         if selected != songWidget.checkButton.get_active():
             songWidget.checkButton.set_active(selected)
 
@@ -761,3 +793,42 @@ class PlaylistDialog():
                                  self.view.get_columns()[0], False)
             self.view.row_activated(self.model.get_path(new_iter),
                                     self.view.get_columns()[0])
+
+
+class CellRendererClickablePixbuf(Gtk.CellRendererPixbuf):
+
+    __gsignals__ = {'clicked': (GObject.SIGNAL_RUN_LAST, GObject.TYPE_NONE,
+                                (GObject.TYPE_STRING,))}
+    __gproperties__ = {
+        'show_star': (GObject.TYPE_BOOLEAN, 'Show star', 'show star', False, GObject.PARAM_READWRITE)}
+
+    starIcon = 'starred-symbolic'
+    nonStarIcon = 'non-starred-symbolic'
+
+    def __init__(self, view, hidden=False, *args, **kwargs):
+        Gtk.CellRendererPixbuf.__init__(self, *args, **kwargs)
+        self.set_property('mode', Gtk.CellRendererMode.ACTIVATABLE)
+        self.set_property('xpad', 32)
+        self.set_property('icon_name', self.nonStarIcon)
+        self.view = view
+        self.hidden = hidden
+        self.show_star = False
+
+    def do_activate(self, event, widget, path, background_area, cell_area, flags):
+        self.show_star = False
+        self.emit('clicked', path)
+
+    def do_get_property(self, property):
+        if property.name == 'show-star':
+            return self.show_star
+
+    def do_set_property(self, property, value):
+        if property.name == 'show-star':
+            if self.show_star:
+                self.set_property('icon_name', self.starIcon)
+            else:
+                if not self.hidden:
+                    self.set_property('icon_name', self.nonStarIcon)
+                else:
+                    self.set_property('icon_name', '')
+            self.show_star = value
