@@ -30,10 +30,11 @@
 # delete this exception statement from your version.
 
 
-from gi.repository import Gtk, Gdk, Gio, GLib, Tracker
+from gi.repository import Gtk, Gdk, Gio, GLib
 from gi.repository import Gd
 from gettext import gettext as _, ngettext
 
+from gnomemusic import TrackerWrapper
 from gnomemusic.toolbar import Toolbar, ToolbarState
 from gnomemusic.player import Player, SelectionToolbar
 from gnomemusic.query import Query
@@ -45,13 +46,8 @@ from gnomemusic import log
 import logging
 logger = logging.getLogger(__name__)
 
-try:
-    tracker = Tracker.SparqlConnection.get(None)
-except Exception as e:
-    from sys import exit
-    logger.error("Cannot connect to tracker, error '%s'\Exiting" % str(e))
-    exit(1)
-playlist = Playlists.get_default(tracker)
+tracker = TrackerWrapper().tracker
+playlist = Playlists.get_default()
 
 
 class Window(Gtk.ApplicationWindow):
@@ -309,7 +305,7 @@ class Window(Gtk.ApplicationWindow):
         self._stack.get_visible_child().queue_draw()
 
     @log
-    def _init_notification(self):
+    def _init_playlist_removal_notification(self):
         self.notification = Gd.Notification()
         self.notification.set_timeout(20)
 
@@ -326,14 +322,27 @@ class Window(Gtk.ApplicationWindow):
         self.notification.show_all()
         self._overlay.add_overlay(self.notification)
 
-        self.notification.connect("dismissed", self._notification_dismissed)
+        self.notification.connect("dismissed", self._playlist_removal_notification_dismissed)
         undo_button.connect("clicked", self._undo_deletion)
 
     @log
-    def _notification_dismissed(self, widget):
+    def _playlist_removal_notification_dismissed(self, widget):
         if self.views[3].really_delete:
             Views.playlists.delete_playlist(self.views[3].pl_todelete)
 
+    @log
+    def _init_loading_notification(self):
+        self.notification = Gd.Notification()
+        grid = Gtk.Grid()
+        grid.set_column_spacing(8)
+        self.notification.add(grid)
+        spinner = Gtk.Spinner()
+        grid.add(spinner)
+        grid.add(Gtk.Label.new(_("Loading")))
+        spinner.start()
+        self.notification.show_all()
+        self._overlay.add_overlay(self.notification)
+
     @log
     def _undo_deletion(self, widget):
         self.views[3].really_delete = False
@@ -359,7 +368,7 @@ class Window(Gtk.ApplicationWindow):
         else:
             if (event.keyval == Gdk.KEY_Delete):
                 if self._stack.get_visible_child() == self.views[3]:
-                    self._init_notification()
+                    self._init_playlist_removal_notification()
                     self.views[3].delete_selected_playlist()
             # Close search bar after Esc is pressed
             if event.keyval == Gdk.KEY_Escape:
